package me.msoucy.ptures.model

class Engine(private vararg val creatures : Pair<Creature, Int>) {
    init {
        assert(creatures.isNotEmpty())
    }

    private var currentCreature = 0

    val attacker : Creature get() {
        return creatures[currentCreature].first
    }

    fun resolveTurn() {
        val activeCreatures = creatures.withIndex()
                .filter { (_, c) -> !c.first.hasStatus<KnockedOut>() }
                .sortedBy { (_, c) -> c.first.spd }
        // Get the moves each creature will use this turn
        val moves = activeCreatures.map { (_, _) -> Pair(Ignore, 0) }
        for ((i, c) in activeCreatures) {
            // Resolve move
            val (skill, target) = moves[i]
            currentCreature = i
            for (step in skill.damageSteps) {
                for (t in getTargetList(step.target, target)) {
                    val (targetCreature, _) = creatures[t]
                    if (attacker.hits(targetCreature, step)) {
                        targetCreature.apply(step, attacker)
                        step.applyStatus(targetCreature)
                    }
                }
            }
            for(step in skill.postSteps) {
                for (t in getTargetList(step.target, target)) {
                    val (targetCreature, _) = creatures[t]
                    step.apply(targetCreature)
                }
            }
        }
    }

    private fun getTargetList(target : Target, selected : Int) : List<Int> {
        return when(target) {
            Target.Self -> listOf(currentCreature)
            Target.Selected -> listOf(selected)
            Target.Others -> creatures.indices.filter { it != currentCreature }
            Target.Opponents -> creatures.indices.filter { creatures[it].second != creatures[currentCreature].second }
            Target.All -> creatures.indices.toList()
        }
    }
}